﻿using FarmatikoData.FarmatikoRepoInterfaces;
using FarmatikoData.Models;
using FarmatikoServices.FarmatikoServiceInterfaces;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace FarmatikoServices.Services
{
    public class Service : IService
    {
        private readonly IRepository _repository;
        public Service(IRepository repository)
        {
            _repository = repository;
        }

        //GET
        public async Task<IQueryable<HealthcareWorker>> GetAllWorkers()
        {
            var Workers = await _repository.GetAllWorkers();
            return Workers;
        }

        public async Task<IQueryable<HealthFacility>> GetFacilities()
        {
            var Facilities = await _repository.GetFacilities();
            return Facilities;
        }

        public async Task<HealthFacility> GetFacility(int id)
        {
            var Facility = await _repository.GetFacility(id);
            return Facility;
        }

        public async Task<Medicine> GetMedicine(int id)
        {
            var Medicine = await _repository.GetMedicine(id);
            return Medicine;
        }

        public async Task<IQueryable<Medicine>> GetMedicines()
        {
            var Medicines = await _repository.GetMedicines();
            return Medicines;
        }

        public async Task<Pandemic> GetPandemic()
        {
            var Pandemic = await _repository.GetPandemic();
            return Pandemic;
        }

        public async Task<IQueryable<Pharmacy>> GetPharmacies()
        {
            var Pharmacies = await _repository.GetPharmacies();
            return Pharmacies;
        }

        public async Task<Pharmacy> GetPharmacy(int id)
        {
            var Pharmacy = await _repository.GetPharmacy(id);
            return Pharmacy;
        }

        public async Task<HealthcareWorker> GetWorker(int id)
        {
            var Worker = await _repository.GetWorker(id);
            return Worker;
        }

        public async Task<IQueryable<HealthFacility>> SearchFacilities(string query)
        {
            var SearchQuery = await _repository.SearchFacilities(query);
            return SearchQuery;
        }

        public async Task<IQueryable<Medicine>> SearchMedicines(string query)
        {
            var SearchQuery = await _repository.SearchMedicines(query);
            return SearchQuery;
        }

        public async Task<IQueryable<Pharmacy>> SearchPharmacies(string query)
        {
            var SearchQuery = await _repository.SearchPharmacies(query);
            return SearchQuery;
        }

        public async Task<IQueryable<HealthcareWorker>> SearchWorkers(string query)
        {
            var SearchQuery = await _repository.SearchWorkers(query);
            return SearchQuery;
        }


        //POST (ADD NEW OBJECTS)
        //za json(Sys updateer)
        public async Task AddFacility(HealthFacility healthFacilities)
        {
            if (healthFacilities != null)
                await _repository.AddFacility(healthFacilities);
            throw new Exception("Facility is null");
        }
        //za json(Sys updateer)
        public async Task AddMedicines(Medicine medicine)
        {
            if (medicine != null)
                await _repository.AddMedicines(medicine);
            throw new Exception("Medicine is null");
        }
        //za json(Sys updateer)
        public async Task AddPandemic(Pandemic pandemic)
        {
            if (pandemic != null)
                await _repository.AddPandemic(pandemic);
            throw new Exception("Pandemic is null");
        }
        // Samo PharmacyHead i Admin imaat pristap
        public async Task AddPharmacy(Pharmacy pharmacy)
        {
            if (pharmacy != null)
                await _repository.AddPharmacy(pharmacy);
            throw new Exception("Pharmacy is null");
        }
        // Ovaa kontrola ja ima samo admin
        public async Task AddPharmacyHead(PharmacyHead pharmacyHead)
        {
            if (pharmacyHead != null)
            {
                var Medicines = await _repository.GetMedicines();
                foreach (var med in Medicines)
                {
                    MedicineList medicine = new MedicineList()
                    {
                        Medicine = med,
                        HasMedicine = false
                    };
                    pharmacyHead.MedicineLists.Add(medicine);
                }
                await _repository.AddPharmacyHead(pharmacyHead);
            }
            throw new Exception("PharmacyHead is null");
        }
        //za json(Sys updateer)
        public async Task AddWorker(HealthcareWorker worker)
        {
            if (worker != null)
                await _repository.AddWorker(worker);
            throw new Exception("Worker is null");
        }

        //za json(Sys updateer)
        public async Task UpdateFacility(HealthFacility healthFacilities)
        {
            if (healthFacilities != null)
                await _repository.UpdateFacility(healthFacilities);
            throw new Exception("Facility is null");
        }
        //PharmacyHead
        public async Task RemoveMedicine(Medicine medicine)
        {
            if (medicine != null)
                await _repository.RemoveMedicine(medicine);
            throw new Exception("Medicine is null");
        }
        //PharmacyHead
        public async Task UpdateMedicine(Medicine medicine)
        {
            if (medicine != null)
                await _repository.UpdateMedicine(medicine);
            throw new Exception("Medicine is null");
        }
        //za json(Sys updateer)
        public async Task UpdatePandemic(Pandemic pandemic)
        {
            if (pandemic != null)
                await _repository.UpdatePandemic(pandemic);
            throw new Exception("Pandemic is null");
        }
        //PharmacyHead
        public async Task RemovePharmacy(Pharmacy pharmacy)
        {
            if (pharmacy != null)
                await _repository.RemovePharmacy(pharmacy);
            throw new Exception("Pharmacy is null");
        }
        //PharamcyHead
        public async Task UpdatePharmacy(Pharmacy pharmacy)
        {
            if (pharmacy != null)
                await _repository.UpadatePharmacy(pharmacy);
            throw new Exception("Pharmacy is null");
        }
        //za json(Sys updateer)
        public async Task UpdateWorker(HealthcareWorker worker)
        {
            if (worker != null)
                await _repository.UpdateWorker(worker);
            throw new Exception("Worker is null");
        }

        public async Task RemovePharmacyHead(PharmacyHead pharmacyHead)
        {
            if (pharmacyHead != null)
            {
                pharmacyHead.DeletedOn = DateTime.UtcNow;
                await _repository.RemovePharmacyHead(pharmacyHead);
            }
            else throw new Exception("PharmacyHead is null.");
        }

        public async Task<HealthFacility> GetFacilityJSON(string healthFacility)
        {
            if (healthFacility != null)
                return await _repository.GetFacilityJSON(healthFacility);
            return null;
        }

        //PUT (EDIT OBJECTS)


        //DELETE

    }
}
